@@ -68,7 +68,7 @@ linux_name_from_sysfs(int fd, char **out)
 
 	ret = fstat(fd, &buffer);
 	if (ret)
-		return ret;
+		return -EINVAL;
 
 	if (!S_ISCHR(buffer.st_mode))
 		return -EINVAL;
@@ -114,6 +114,10 @@ linux_from_sysfs(int fd, struct kms_driver **out)
 #ifdef HAVE_RADEON
 	else if (!strcmp(name, "radeon"))
 		ret = radeon_create(fd, out);
+#endif
+#ifdef HAVE_EXYNOS
+	else if (!strcmp(name, "exynos"))
+		ret = exynos_create(fd, out);
 #endif
 	else
 		ret = -ENOSYS;
